@@ -2,6 +2,7 @@
 
 from qds_sdk.qubole import Qubole
 from qds_sdk.commands import *
+from qds_sdk.cluster import *
 import qds_sdk.exception
 
 import os
@@ -13,14 +14,15 @@ from optparse import OptionParser
 log = logging.getLogger("qds")
 
 usage_str = ("Usage: \n"
-             "qds [options] "
-             "<hivecmd|hadoopcmd|pigcmd|shellcmd> <submit|run|check|cancel> [args .. ]\n"
-             "\tsubmit [cmd-specific-args .. ] : submit cmd & print id \n"
-             "\trun [cmd-specific-args .. ] : submit cmd & wait. print results \n"
-             "\tcheck <id> : print the cmd object for this Id\n"
-             "\tcancel <id> : cancels the cmd with this Id\n"
-             "\tgetresult <id> : get the results for the cmd with this Id\n"
-             "\tgetlog <id> : get the logs for the cmd with this Id\n")
+             "qds [options] <CmdArgs | hadoop_cluster check>\n"
+             "\nCmdArgs:\n" +
+             "  <hivecmd|hadoopcmd|pigcmd|shellcmd> <submit|run|check|cancel> [args .. ]\n"
+             "  submit [cmd-specific-args .. ] : submit cmd & print id \n"
+             "  run [cmd-specific-args .. ] : submit cmd & wait. print results \n"
+             "  check <id> : print the cmd object for this Id\n"
+             "  cancel <id> : cancels the cmd with this Id\n"
+             "  getresult <id> : get the results for the cmd with this Id\n"
+             "  getlog <id> : get the logs for the cmd with this Id\n")
 
 def usage(parser=None):
     if parser is None:
@@ -44,7 +46,7 @@ def submitaction(cmdclass, args):
 def _getresult(cmdclass, cmd):
     if (Command.is_success(cmd.status)):
         log.info("Fetching results for %s, Id: %s" % (cmdclass.__name__, cmd.id))
-        cmd.get_results(sys.stdout)
+        cmd.get_results(sys.stdout, delim='\t')
         return 0
     else:
         log.error("Cannot fetch results - command Id: %s failed with status: %s" % (cmd.id, cmd.status))
@@ -95,7 +97,6 @@ def cmdmain(cmd, args):
     cmdclassname = cmd[0].upper() + cmd[1:] + "Command"
     cmdclass = globals()[cmdclassname]
 
-
     actionset = set(["submit", "run", "check", "cancel", "getresult", "getlog"])
     if len(args) < 1:
         sys.stderr.write("missing argument containing action\n")
@@ -109,6 +110,28 @@ def cmdmain(cmd, args):
 
     return globals()[action + "action"](cmdclass, args)
 
+def clustercheckaction(clusterclass, args):
+    name = args.pop(0) if (len(args) >= 1) else None
+    o = clusterclass.find(name=name)
+    print str(o)
+    return 0
+
+def clustermain(dummy, args):
+    clusterclass = HadoopCluster
+    actionset = set(["check"])
+
+    if len(args) < 1:
+        sys.stderr.write("missing argument containing action\n")
+        usage()
+    
+    action = args.pop(0)
+    if action not in actionset:
+        sys.stderr.write("action must be one of <%s>\n" % "|".join(actionset))
+        usage()
+
+
+    return globals()["cluster" + action + "action"](clusterclass, args)
+
 
 def main():
 
@@ -171,19 +194,23 @@ def main():
 
     if len(args) < 1:
         sys.stderr.write("Missing first argument containing command type\n")
-        usage()
+        usage(optparser)
 
-    cmdset = set(["hive", "pig", "hadoop", "shell"])
     cmdsuffix = "cmd"
+    cmdset = set([x + cmdsuffix for x in ["hive", "pig", "hadoop", "shell"]])
 
-    cmd = args.pop(0)
 
-    if ((cmd.find(cmdsuffix) != len(cmd)-3) or
-        (cmd[:cmd.find(cmdsuffix)] not in cmdset)):
-        sys.stderr.write("First command must be one of <%s>\n" % "|".join(cmdset))
-        usage()
-        
-    return cmdmain(cmd[:cmd.find(cmdsuffix)], args)
+    a0 = args.pop(0)
+
+    if (a0 in cmdset):
+        return cmdmain(a0[:a0.find(cmdsuffix)], args)
+
+    if (a0 == "hadoop_cluster"):
+        return clustermain(a0, args)
+
+    sys.stderr.write("First command must be one of <%s>\n" % 
+                     "|".join(cmdset.union(["hadoop_cluster"])))
+    usage(optparser)
 
 
 if __name__ == '__main__':
