@@ -14,7 +14,6 @@ class Error(Exception):
         Exception.__init__(self, request.text)
         self.request = request
 
-
 class ServerError(Error):
     """An error caused by the server."""
     # HTTP error code 5xx (500..599)
@@ -42,6 +41,12 @@ class ResourceConflict(ClientError):
     pass
 
 
+class RetryWithDelay(ClientError):
+    """An error raised when a resource must be retried."""
+    # 449 Retry With 
+    pass
+
+
 class ResourceInvalid(ClientError):
     """An error raised when a resource is invalid."""
     # 422 Resource Invalid
